from PyDictAPI import Finder, Translate
import requests
F = Finder(jsonify=True)
T = Translate()


#print(T.languages_help())
#print(T.translateItems("help", "hi"))
'''
print(F.findMeanings("help"))

print(F.findSynonyms("help"))
print(F.findAntonyms("help"))
print(F.findUsage("help"))
'''

input()